module Gitmain
  class Blob
    attr_reader :id, :name, :path, :repository

    def initialize(repository, id, path = nil, name = nil)
      raise ArgumentError, 'provide a valid repository' unless repository.kind_of?(Gitmain::Repository)
      raise ArgumentError, 'provide a valid blob id' unless id.to_s.match(/\A[0-9a-f]{40}\z/)
      @repository = repository
      @id = id
      @path = path
      @name = name
    end

    def changed_attributes
      @changed_attributes ||= {}
    end

    def content
      @content ||= _blob&.content
    end

    def content=(value)
      changed_attributes[:content] = content
      @content = value
    end

    def file_name
      File.join(*[path, name].map(&:to_s).reject(&:empty?))
    end

    private

    def _blob
      @_blob ||= repository.rugged(id)
    end
  end
end
