module Gitmain
  class Repository
    attr_reader :branch, :name

    def self.default_branch(value = nil)
      @default_branch = value unless value.to_s.empty?

      if @default_branch.to_s.empty?
        'master'
      else
        @default_branch
      end
    end

    def self.dirname(value = nil)
      @dirname = value unless value.to_s.empty?

      if @dirname.to_s.empty?
        raise ArgumentError, 'provide a value to set @dirname'
      else
        @dirname
      end
    end

    def self.init(name)
      raise ArgumentError, 'provide a valid name' unless name.match(/\A[\w\-]+\z/)
      Rugged::Repository.init_at(File.join(dirname, "#{name}.git"), :bare)
      new(name)
    end

    def initialize(name, branch = self.class.default_branch)
      raise ArgumentError, 'provide a valid name' unless name.match(/\A[\w\-]+\z/)
      raise ArgumentError, 'provide a valid branch' unless branch.match(/\A[\w\-]+\z/)

      begin
        @_repo = Rugged::Repository.new(File.join(self.class.dirname, "#{name}.git"))
      rescue
        raise ArgumentError, 'could not load repository'
      end

      unless @_repo.bare?
        raise ArgumentError, 'must be a bare repository'
      end

      if @_repo.branches.each_name.include?(branch)
        @_branch = @_repo.branches[branch]
      elsif !@_repo.empty?
        raise ArgumentError, 'must provide an existing branch'
      end

      @name = name
      @branch = branch
    end

    def commit(klass: Gitmain::Commit)
      klass.new(self, commit_id) if commit_id
    end

    def commit_id
      @_branch&.target&.oid
    end

    def checkout(branch)
      if @_repo.branches.each_name.include?(branch)
        @_branch = @_repo.branches[branch]
        @branch = branch
      elsif @_repo.branches.each_name.include?(self.class.default_branch)
        @_branch = @_repo.branches.create(branch, self.class.default_branch)
        @branch = branch
      else
        raise ArgumentError, "default branch not yet setup"
      end
    end

    def rename(name)
      FileUtils.mv(File.join(self.class.dirname, "#@name.git"), File.join(self.class.dirname, "#{name}.git"))
      @_repo = Rugged::Repository.new(File.join(self.class.dirname, "#{name}.git"))
      @name = name
    end

    def rugged(id)
      @_repo&.lookup(id)
    end

    def setup?
      @_repo && !@_repo.empty? && @_repo.branches[self.class.default_branch]
    end

    def tree(klass: Gitmain::Tree)
      klass.new(self, tree_id) if tree_id
    end

    def tree_id
      @_branch&.target&.tree_id
    end
  end
end
