module Gitmain
  class Tree
    attr_reader :id, :name, :path, :repository

    def initialize(repository, id, path = nil, name = nil)
      raise ArgumentError, 'provide a valid repository' unless repository.kind_of?(Gitmain::Repository)
      raise ArgumentError, 'provide a valid tree id' unless id.to_s.match(/\A[0-9a-f]{40}\z/)
      @repository = repository
      @id = id
      @path = path
      @name = name
    end

    def blobs(klass: Gitmain::Blob, recursive: false)
      if recursive
        _tree.walk(:postorder).map do |root, object|
          if object[:type] == :blob
            blob_path = File.join(*[path, name, root].map(&:to_s).reject(&:empty?))
            blob_path.sub!(/\/\z/, '')
            blob_path = nil if blob_path.empty?
            klass.new(repository, object[:oid], blob_path, object[:name])
          end
        end.compact
      else
        _tree.each.map do |object|
          if object[:type] == :blob
            blob_path = File.join(*[path, name].map(&:to_s).reject(&:empty?))
            blob_path.sub!(/\/\z/, '')
            blob_path = nil if blob_path.empty?
            klass.new(repository, object[:oid], blob_path, object[:name])
          end
        end.compact
      end
    end

    def file_name
      File.join(*[path, name].map(&:to_s).reject(&:empty?))
    end

    def trees(klass: Gitmain::Tree, recursive: false)
      if recursive
        _tree.walk(:postorder).map do |root, object|
          if object[:type] == :tree
            tree_path = File.join(*[path, name, root].map(&:to_s).reject(&:empty?))
            tree_path.sub!(/\/\z/, '')
            tree_path = nil if tree_path.empty?
            klass.new(repository, object[:oid], tree_path, object[:name])
          end
        end.compact
      else
        _tree.each.map do |object|
          if object[:type] == :tree
            tree_path = File.join(*[path, name].map(&:to_s).reject(&:empty?))
            tree_path.sub!(/\/\z/, '')
            tree_path = nil if tree_path.empty?
            klass.new(repository, object[:oid], tree_path, object[:name])
          end
        end.compact
      end
    end

    private

    def _tree
      @_tree ||= repository.rugged(id)
    end
  end
end
